@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-topic-action.c,v 1.35 2004/01/28 19:00:37 chpe Exp $
+ *  $Id: ephy-topic-action.c,v 1.51 2004/07/31 11:58:56 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -24,24 +24,41 @@
 
 #include <gtk/gtktoolitem.h>
 #include <glib/gi18n.h>
+#include <libgnomevfs/gnome-vfs-uri.h>
+#include <string.h>
 
 #include "ephy-topic-action.h"
 #include "ephy-node-common.h"
 #include "ephy-bookmarks.h"
+#include "ephy-bookmarksbar.h"
 #include "ephy-favicon-cache.h"
 #include "ephy-shell.h"
 #include "ephy-debug.h"
+#include "ephy-dnd.h"
 #include "ephy-gui.h"
 #include "ephy-string.h"
+#include "ephy-marshal.h"
 
 static void ephy_topic_action_init       (EphyTopicAction *action);
 static void ephy_topic_action_class_init (EphyTopicActionClass *class);
 
 #define EPHY_TOPIC_ACTION_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_TOPIC_ACTION, EphyTopicActionPrivate))
 
+static GtkTargetEntry drag_targets[] =
+{
+	{ EPHY_DND_TOPIC_TYPE, 0, 0 }
+};
+static int n_drag_targets = G_N_ELEMENTS (drag_targets);
+
 struct EphyTopicActionPrivate
 {
-	int topic_id;
+	guint topic_id;
+	EphyNode *topic_node;
+
+	guint motion_handler;
+	guint release_handler;
+	gint drag_x;
+	gint drag_y;
 };
 
 enum
@@ -52,7 +69,9 @@ enum
 
 enum
 {
-	GO_LOCATION,
+	OPEN,
+	OPEN_IN_TAB,
+	OPEN_IN_TABS,
 	LAST_SIGNAL
 };
 
@@ -103,6 +122,7 @@ create_tool_item (GtkAction *action)
 	gtk_container_add (GTK_CONTAINER (item), hbox);
 
 	button = gtk_toggle_button_new ();
+	gtk_widget_add_events (GTK_WIDGET (button), GDK_BUTTON1_MOTION_MASK);
 	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
 	gtk_widget_show (button);
 	gtk_container_add (GTK_CONTAINER (hbox), button);
@@ -127,6 +147,7 @@ create_tool_item (GtkAction *action)
 static void
 menu_deactivate_cb (GtkMenuShell *ms, GtkWidget *button)
 {
+	g_object_set_data (G_OBJECT (button), "popup", NULL);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
 	gtk_button_released (GTK_BUTTON (button));
 }
@@ -140,8 +161,17 @@ menu_activate_cb (GtkWidget *item, GtkAction *action)
 	node = g_object_get_data (G_OBJECT (item), "node");
 	location = ephy_node_get_property_string
 		(node, EPHY_NODE_BMK_PROP_LOCATION);
-	g_signal_emit (action, ephy_topic_action_signals[GO_LOCATION],
-		       0, location);
+
+	if (ephy_gui_is_middle_click ())
+	{
+		g_signal_emit (action, ephy_topic_action_signals[OPEN_IN_TAB],
+			       0, location, FALSE);
+	}
+	else
+	{
+		g_signal_emit (action, ephy_topic_action_signals[OPEN],
+			       0, location);
+	}
 }
 
 static void
@@ -215,6 +245,16 @@ sort_bookmarks (gconstpointer a, gconstpointer b)
 
 #define MAX_LENGTH 32
 
+static gboolean
+can_open_in_tabs (EphyTopicAction *action)
+{
+	GPtrArray *children;
+
+	children = ephy_node_get_children (action->priv->topic_node);
+
+	return (children->len > 1);
+}
+
 static void
 append_bookmarks_menu (EphyTopicAction *action, GtkWidget *menu, EphyNode *node, gboolean show_empty)
 {
@@ -234,6 +274,8 @@ append_bookmarks_menu (EphyTopicAction *action, GtkWidget *menu, EphyNode *node,
 		gtk_widget_set_sensitive (item, FALSE);
 		gtk_widget_show (item);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+		return;
 	}
 	else
 	{
@@ -295,6 +337,120 @@ append_bookmarks_menu (EphyTopicAction *action, GtkWidget *menu, EphyNode *node,
 	}
 }
 
+static void
+open_in_tabs_activate_cb (GtkWidget *menu, EphyTopicAction *action)
+{
+	GList *uri_list = NULL;
+	GPtrArray *children;
+	int i;
+
+	children = ephy_node_get_children (action->priv->topic_node);
+	for (i = 0; i < children->len; i++)
+	{
+		const char *address;
+		EphyNode *child;
+
+		child = g_ptr_array_index (children, i);
+		address = ephy_node_get_property_string
+			(child, EPHY_NODE_BMK_PROP_LOCATION);
+		uri_list = g_list_append (uri_list, g_strdup (address));
+	}
+
+	g_signal_emit (action, ephy_topic_action_signals[OPEN_IN_TABS],
+		       0, uri_list);
+	g_list_foreach (uri_list, (GFunc)g_free, NULL);
+	g_list_free (uri_list);
+}
+
+static int
+get_item_position (GtkWidget *widget, gboolean *last)
+{
+	GtkWidget *item, *toolbar;
+	int index;
+
+	item = gtk_widget_get_ancestor (widget, GTK_TYPE_TOOL_ITEM);
+	g_return_val_if_fail (item != NULL, -1);
+
+	toolbar = gtk_widget_get_ancestor (widget, GTK_TYPE_TOOLBAR);
+	g_return_val_if_fail (toolbar != NULL, -1);
+
+	index = gtk_toolbar_get_item_index (GTK_TOOLBAR (toolbar),
+				            GTK_TOOL_ITEM (item));
+	if (last)
+	{
+		int n_items;
+
+		n_items = gtk_toolbar_get_n_items (GTK_TOOLBAR (toolbar));
+		*last = (index == n_items - 1);
+	}
+
+	return index;
+}
+
+static void
+remove_from_model (GtkWidget *widget)
+{
+	EphyBookmarks *bookmarks;
+	EggToolbarsModel *model;
+	int pos;
+
+	pos = get_item_position (widget, NULL);
+
+	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
+	model = ephy_bookmarks_get_toolbars_model (bookmarks);
+
+	egg_toolbars_model_remove_item (model, 0, pos);
+}
+
+static void
+move_in_model (GtkWidget *widget, int direction)
+{
+	EphyBookmarks *bookmarks;
+	EggToolbarsModel *model;
+	int pos, new_pos;
+
+	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
+	model = ephy_bookmarks_get_toolbars_model (bookmarks);
+
+	pos = get_item_position (widget, NULL);
+	new_pos = MAX (0, pos + direction);
+
+	egg_toolbars_model_move_item (model, 0, pos, 0, new_pos);
+}
+
+static void
+remove_activate_cb (GtkWidget *menu, GtkWidget *proxy)
+{
+	remove_from_model (proxy);
+}
+
+static void
+move_left_activate_cb (GtkWidget *menu, GtkWidget *proxy)
+{
+	move_in_model (proxy, -1);
+}
+
+static void
+move_right_activate_cb (GtkWidget *menu, GtkWidget *proxy)
+{
+	move_in_model (proxy, +1);
+}
+
+static GtkWidget *
+add_open_in_tabs_menu (EphyTopicAction *action, GtkWidget *menu)
+{
+	GtkWidget *item;
+
+	item = gtk_menu_item_new_with_mnemonic (_("_Open in Tabs"));
+	gtk_widget_show (item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+	g_signal_connect (item, "activate",
+			  G_CALLBACK (open_in_tabs_activate_cb), action);
+
+	return item;
+}
+
 static GtkWidget *
 build_bookmarks_menu (EphyTopicAction *action, EphyNode *node)
 {
@@ -304,6 +460,17 @@ build_bookmarks_menu (EphyTopicAction *action, EphyNode *node)
 
 	append_bookmarks_menu (action, menu, node, TRUE);
 
+	if (can_open_in_tabs (action))
+	{
+		GtkWidget *item;
+
+		item = gtk_separator_menu_item_new ();
+		gtk_widget_show (item);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+		add_open_in_tabs_menu (action, menu);
+	}
+
 	return menu;
 }
 
@@ -341,18 +508,19 @@ sort_topics (gconstpointer a, gconstpointer b)
 }
 
 static GtkWidget *
-build_topics_menu (EphyTopicAction *action, EphyNode *node)
+build_topics_menu (EphyTopicAction *action)
 {
-	GtkWidget *menu, *item, *label;
+	GtkWidget *menu, *item;
 	GPtrArray *children;
 	int i;
 	EphyBookmarks *bookmarks;
-	EphyNode *all, *uncategorized;
+	EphyNode *all, *uncategorized, *node;
 	EphyNodePriority priority;
 	GList *node_list = NULL, *l = NULL;
 
 	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
 	all = ephy_bookmarks_get_bookmarks (bookmarks);
+	node = ephy_bookmarks_get_keywords (bookmarks);
 
 	menu = gtk_menu_new ();
 
@@ -388,9 +556,6 @@ build_topics_menu (EphyTopicAction *action, EphyNode *node)
 
 		item = gtk_image_menu_item_new_with_label (title);
 
-		label = gtk_bin_get_child (GTK_BIN (item));
-		gtk_label_set_use_markup  (GTK_LABEL (label), TRUE);
-
 		gtk_widget_show (item);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
@@ -409,47 +574,114 @@ build_topics_menu (EphyTopicAction *action, EphyNode *node)
 static GtkWidget *
 build_menu (EphyTopicAction *action)
 {
-	EphyNode *node;
-	EphyBookmarks *bookmarks;
-
-	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
-
 	if (action->priv->topic_id == BOOKMARKS_NODE_ID)
 	{
-		LOG ("Build all bookmarks crap menu")
-
-		node = ephy_bookmarks_get_keywords (bookmarks);
-		return build_topics_menu (action, node);
+		return build_topics_menu (action);
 	}
 	else
 	{
-		node = ephy_bookmarks_get_from_id (bookmarks, action->priv->topic_id);
-		return build_bookmarks_menu (action, node);
+		return build_bookmarks_menu (action, action->priv->topic_node);
 	}
 }
 
+static void
+drag_data_get_cb (GtkWidget *widget, GdkDragContext *context,
+		  GtkSelectionData *selection_data, guint info,
+		  guint32 time, EphyTopicAction *action)
+{
+	EphyBookmarks *bookmarks;
+	char *uri;
+
+	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
+	g_return_if_fail (bookmarks != NULL);
+
+	uri = ephy_bookmarks_get_topic_uri (bookmarks, action->priv->topic_node);
+	g_return_if_fail (uri != NULL);
+
+	gtk_selection_data_set (selection_data, selection_data->target, 8,
+				(unsigned char *) uri, strlen (uri));
+
+	g_free (uri);
+}
+
+static void
+drag_data_delete_cb (GtkWidget *widget, GdkDragContext *context,
+		     EphyTopicAction *action)
+{
+	remove_from_model (widget);
+}
+
+static void
+stop_drag_check (EphyTopicAction *action, GtkWidget *widget)
+{
+	if (action->priv->motion_handler)
+	{
+		g_signal_handler_disconnect (widget, action->priv->motion_handler);
+		action->priv->motion_handler = 0;
+
+		g_signal_handler_disconnect (widget, action->priv->release_handler);
+		action->priv->release_handler = 0;
+	}
+}
+
+static gboolean
+check_horizontal_threshold (GtkWidget *widget, gint start_x, gint start_y,
+			    gint current_x, gint current_y)
+{
+	gint drag_threshold;
+
+	g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
+
+	g_object_get (gtk_widget_get_settings (widget),
+		      "gtk-dnd-drag-threshold", &drag_threshold,
+		      NULL);
+
+	return (ABS (current_x - start_x) > drag_threshold &&
+		ABS (current_y - start_y) < drag_threshold);
+}
+
+static gboolean
+drag_motion_cb (GtkWidget *widget, GdkEventMotion *event, EphyTopicAction *action)
+{
+	GtkWidget *button, *event_widget;
+
+	event_widget = gtk_get_event_widget ((GdkEvent*) event);
+	button = GTK_WIDGET (g_object_get_data (G_OBJECT (widget), "button"));
+
+	if (!gtk_widget_is_ancestor (event_widget, widget) &&
+	    check_horizontal_threshold (widget, action->priv->drag_x,
+				        action->priv->drag_y, event->x, event->y))
+	{
+		GtkTargetList *target_list;
+
+		target_list = gtk_target_list_new (drag_targets, n_drag_targets);
+
+		stop_drag_check (action, widget);
+		gtk_menu_popdown (GTK_MENU (widget));
+		gtk_drag_begin (button, target_list, GDK_ACTION_MOVE |
+				GDK_ACTION_COPY, 1, (GdkEvent*)event);
+	}
+
+	return TRUE;
+}
+
 static void
 button_toggled_cb (GtkWidget *button,
 		   EphyTopicAction *action)
 {
-	GtkWidget *menu;
-
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)))
 	{
+		GtkWidget *menu;
+
 		menu = build_menu (action);
 		g_signal_connect (menu, "deactivate",
 				  G_CALLBACK (menu_deactivate_cb), button);
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
 				ephy_gui_menu_position_under_widget,
 				button, 1, gtk_get_current_event_time ());
-	}
-}
 
-static void
-button_pressed_cb (GtkWidget *button,
-		   EphyTopicAction *action)
-{
-	 gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
+		g_object_set_data (G_OBJECT (button), "popup", menu);
+	}
 }
 
 static GtkWidget *
@@ -481,24 +713,118 @@ create_menu_item (GtkAction *action)
 	return menu_item;
 }
 
-static gboolean
-create_menu_proxy (GtkToolItem *item, GtkAction *action)
+static void
+show_context_menu (EphyTopicAction *action, GtkWidget *proxy,
+		   GtkMenuPositionFunc func)
 {
-	GtkWidget *menu_item;
-	char *menu_id;
+	GtkWidget *menu, *item;
+	gboolean last;
 
-	LOG ("create_menu_proxy item %p, action %p", item, action)
+	menu = gtk_menu_new ();
 
-	menu_item = create_menu_item (action);
+	item = add_open_in_tabs_menu (action, menu);
+	gtk_widget_set_sensitive (item, can_open_in_tabs (action));
 
-	menu_id = g_strdup_printf ("ephy-topic-action-%d-menu-id",
-				   EPHY_TOPIC_ACTION (action)->priv->topic_id);
+	item = gtk_separator_menu_item_new ();
+	gtk_widget_show (item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
-	gtk_tool_item_set_proxy_menu_item (item, menu_id, menu_item);
+	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_REMOVE, NULL);
+	gtk_widget_show (item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	g_signal_connect (item, "activate",
+			  G_CALLBACK (remove_activate_cb), proxy);
 
-	g_free (menu_id);
+	item = gtk_separator_menu_item_new ();
+	gtk_widget_show (item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
-	return TRUE;
+	item = gtk_menu_item_new_with_mnemonic (_("Move _Left"));
+	gtk_widget_set_sensitive (item, get_item_position (proxy, NULL) > 0);
+	gtk_widget_show (item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	g_signal_connect (item, "activate",
+			  G_CALLBACK (move_left_activate_cb), proxy);
+
+	item = gtk_menu_item_new_with_mnemonic (_("Move Ri_ght"));
+	get_item_position (proxy, &last);
+	gtk_widget_set_sensitive (item, !last);
+	gtk_widget_show (item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	g_signal_connect (item, "activate",
+			  G_CALLBACK (move_right_activate_cb), proxy);
+
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, func, proxy, 3,
+			gtk_get_current_event_time ());
+}
+
+static void
+popup_menu_cb (GtkWidget *widget, EphyTopicAction *action)
+{
+	if (gtk_widget_get_ancestor (widget, EPHY_TYPE_BOOKMARKSBAR))
+        {
+                show_context_menu (action, widget,
+				   ephy_gui_menu_position_under_widget);
+        }
+}
+
+static gboolean
+button_release_cb (GtkWidget *widget,
+                   GdkEventButton *event,
+		   EphyTopicAction *action)
+{
+	if (event->button == 1)
+	{
+		stop_drag_check (action, widget);
+
+		if (GTK_IS_TOGGLE_BUTTON (widget))
+		{
+			gtk_toggle_button_set_active
+				(GTK_TOGGLE_BUTTON (widget), FALSE);
+		}
+	}
+
+	return FALSE;
+}
+
+static gboolean
+button_press_cb (GtkWidget *widget,
+		 GdkEventButton *event,
+		 EphyTopicAction *action)
+{
+	if (event->button == 1 &&
+	    gtk_widget_get_ancestor (widget, EPHY_TYPE_BOOKMARKSBAR))
+	{
+		if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
+		{
+			GtkWidget *menu;
+
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
+			menu = g_object_get_data (G_OBJECT (widget), "popup");
+			g_return_val_if_fail (menu != NULL, FALSE);
+
+			g_object_set_data (G_OBJECT (menu), "button", widget);
+
+			action->priv->drag_x = event->x;
+			action->priv->drag_y = event->y;
+			action->priv->motion_handler = g_signal_connect
+				(menu, "motion_notify_event",
+				 G_CALLBACK (drag_motion_cb), action);
+			action->priv->release_handler = g_signal_connect
+				(menu, "button_release_event",
+				 G_CALLBACK (button_release_cb), action);
+		
+			return TRUE;
+		}
+	}
+	else if (event->button == 3 &&
+	         gtk_widget_get_ancestor (widget, EPHY_TYPE_BOOKMARKSBAR))	
+	{
+		show_context_menu (action, widget, NULL);
+		return TRUE;
+	}
+
+	return FALSE;
 }
 
 static void
@@ -516,34 +842,49 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 
 	if (GTK_IS_TOOL_ITEM (proxy))
 	{
-		g_signal_connect_object (proxy, "create_menu_proxy",
-					 G_CALLBACK (create_menu_proxy),
-					 action, 0);
-
 		button = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "button"));
 		g_signal_connect (button, "toggled",
 				  G_CALLBACK (button_toggled_cb), action);
-
-		/* We want the menu to popup up on mouse down */
-		g_signal_connect (button, "pressed",
-				  G_CALLBACK (button_pressed_cb), action);
+		g_signal_connect (button, "popup_menu",
+				  G_CALLBACK (popup_menu_cb), action);
+		g_signal_connect (button, "button-press-event",
+				  G_CALLBACK (button_press_cb), action);
+		g_signal_connect (button, "button-release-event",
+				  G_CALLBACK (button_release_cb), action);
+		g_signal_connect (button, "drag_data_get",
+				  G_CALLBACK (drag_data_get_cb), action);
+		g_signal_connect (button, "drag_data_delete",
+				  G_CALLBACK (drag_data_delete_cb), action);
 	}
 }
 
+static void
+ephy_topic_action_set_topic_id (EphyTopicAction *action, guint id)
+{
+	EphyBookmarks *bookmarks;
+	EphyNode *node;
+
+	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
+	node = ephy_bookmarks_get_from_id (bookmarks, id);
+
+	action->priv->topic_node = node;
+	action->priv->topic_id = id;
+}
+
 static void
 ephy_topic_action_set_property (GObject *object,
                                 guint prop_id,
                                 const GValue *value,
                                 GParamSpec *pspec)
 {
-	EphyTopicAction *bmk;
+	EphyTopicAction *topic;
 
-	bmk = EPHY_TOPIC_ACTION (object);
+	topic = EPHY_TOPIC_ACTION (object);
 
 	switch (prop_id)
 	{
 		case PROP_TOPIC_ID:
-			bmk->priv->topic_id = g_value_get_int (value);
+			ephy_topic_action_set_topic_id (topic, g_value_get_int (value));
 			break;
 	}
 }
@@ -582,17 +923,40 @@ ephy_topic_action_class_init (EphyTopicActionClass *class)
 	object_class->set_property = ephy_topic_action_set_property;
 	object_class->get_property = ephy_topic_action_get_property;
 
-	ephy_topic_action_signals[GO_LOCATION] =
-                g_signal_new ("go_location",
+	ephy_topic_action_signals[OPEN] =
+                g_signal_new ("open",
                               G_OBJECT_CLASS_TYPE (object_class),
                               G_SIGNAL_RUN_FIRST,
-                              G_STRUCT_OFFSET (EphyTopicActionClass, go_location),
+                              G_STRUCT_OFFSET (EphyTopicActionClass, open),
                               NULL, NULL,
                               g_cclosure_marshal_VOID__STRING,
                               G_TYPE_NONE,
                               1,
 			      G_TYPE_STRING);
 
+	ephy_topic_action_signals[OPEN_IN_TAB] =
+                g_signal_new ("open_in_tab",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (EphyTopicActionClass, open_in_tab),
+                              NULL, NULL,
+                              ephy_marshal_VOID__STRING_BOOLEAN,
+                              G_TYPE_NONE,
+                              2,
+			      G_TYPE_STRING,
+			      G_TYPE_BOOLEAN);
+
+	ephy_topic_action_signals[OPEN_IN_TABS] =
+                g_signal_new ("open_in_tabs",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (EphyTopicActionClass, open_in_tabs),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__POINTER,
+                              G_TYPE_NONE,
+                              1,
+			      G_TYPE_POINTER);
+
 	g_object_class_install_property (object_class,
                                          PROP_TOPIC_ID,
                                          g_param_spec_int ("topic_id",
@@ -634,9 +998,12 @@ sync_topic_properties (GtkAction *action, EphyNode *bmk)
 }
 
 static void
-topic_child_changed_cb (EphyNode *node, EphyNode *child, GtkAction *action)
+topic_child_changed_cb (EphyNode *node,
+			EphyNode *child,
+			guint property_id,
+			GtkAction *action)
 {
-	gulong id;
+	guint id;
 
 	id = EPHY_TOPIC_ACTION (action)->priv->topic_id;
 
